package com.ruoyi.project.production.mapper;

import java.util.List;
import com.ruoyi.framework.aspectj.lang.annotation.DataScope;
import com.ruoyi.project.plant.domain.TMtdDaily;
import com.ruoyi.project.production.domain.TPrdRecord;

/**
 * 生产记录Mapper接口
 *
 * @author ssy
 * @date 2024-10-08
 */
public interface TPrdRecordMapper
{
    /**
     * 查询生产记录
     *
     * @param id 生产记录ID
     * @return 生产记录
     */
    public TPrdRecord selectTPrdRecordById(Long id);

    /**
     * 查询生产记录列表
     *
     * @param tPrdRecord 生产记录
     * @return 生产记录集合
     */
    public List<TPrdRecord> selectTPrdRecordList(TPrdRecord tPrdRecord);

    /**
     * 新增生产记录
     *
     * @param tPrdRecord 生产记录
     * @return 结果
     */
    public int insertTPrdRecord(TPrdRecord tPrdRecord);

    /**
     * 修改生产记录
     *
     * @param tPrdRecord 生产记录
     * @return 结果
     */
    public int updateTPrdRecord(TPrdRecord tPrdRecord);

    /**
     * 删除生产记录
     *
     * @param id 生产记录ID
     * @return 结果
     */
    public int deleteTPrdRecordById(Long id);

    /**
     * 批量删除生产记录
     *
     * @param ids 需要删除的数据ID
     * @return 结果
     */
    public int deleteTPrdRecordByIds(Long[] ids);

    void deleteTPrdRecordByDate(TPrdRecord tPrdRecord);
}
